/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.wsrf.client;

import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Properties;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.globus.axis.util.Util;
import org.apache.axis.AxisFault;

import javax.xml.rpc.ServiceException;

import java.net.MalformedURLException;

import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.authorization.IdentityAuthorization;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;

public class CheckEnvironment  {

    public static String JWSCORE_VERSION_SERVICE = 
        "wsrf/services/JWSCoreVersion";
    public static String AXIS_VERSION_SERVICE = "wsrf/services/Version";

    private static List gtComponents = new ArrayList();
    private static List otherComponents = new ArrayList();

    private static String OPTIONS = 
        "globus-check-environment [-help] [endpoint]\n"
        + "-help Returns the help string.\n"
        + "-s    Optional endpoint for remote version information. \n"
        + "      It should be protocol://host:port\n"
        + "-z    Sets authorization, can be 'self', 'host', 'none' or \n"
        + "      a string specifying the expected identity of the remote \n"
        + "      party. Defaults to no authorization.";

    static {
        // Incase end point is https
        Call.addTransportPackage("org.apache.axis.transport");
        Util.registerTransport();

        // other components
        otherComponents.add(new ComponentVersion("Xerces", 
                                                 "org.apache.xerces.impl.Version", 
                                                 "getVersion"));
        otherComponents.add(new ComponentVersion("Xalan", 
                                                 "org.apache.xalan.Version",
                                                 "getVersion"));

        otherComponents.add(new ComponentVersion("Apache Axis",
                                                 "org.apache.axis.Version",
                                                 "getVersion"));
        
        
        // GT components
        gtComponents.add(new ComponentVersion("Java WS Core",
                                              "org.globus.wsrf.utils.Version",
                                              "getVersion"));
        gtComponents.add(new ComponentVersion("CoG JGlobus",
                                              "org.globus.common.Version",
                                              "getVersion"));
    }

    public static void displayComponentVersions(List components) {
        Iterator iter = components.iterator();
        while(iter.hasNext()) {
            ComponentVersion comp = (ComponentVersion)iter.next();

            System.out.print(comp.getName() + ": ");
            try {
                System.out.println(comp.getVersion());
            } catch (ClassNotFoundException e) {
                System.out.println("N/A");
            } catch (NoSuchMethodException e) {
                System.out.println("N/A");
            } catch (IllegalAccessException e) {
                System.out.println("N/A");
            } catch (InvocationTargetException e) {
                System.out.println("N/A");
            }
        }
    }

    public static void displayRemoteComponentVersions(String endpoint,
                                                      Object authorization) {

        if (endpoint == null) {
            System.out.println("Endpoint not available for remote "
                               + "version information");
            return;
        }

        String gtEndpoint;
        String axisEndpoint;
        
        if (endpoint.endsWith("/")) {
            gtEndpoint = endpoint + JWSCORE_VERSION_SERVICE;
            axisEndpoint = endpoint + AXIS_VERSION_SERVICE;
        } else {
            gtEndpoint = endpoint + "/" + JWSCORE_VERSION_SERVICE;
            axisEndpoint = endpoint + "/" + AXIS_VERSION_SERVICE;
        }

        // Axis version
        System.out.println("Axis Version on remote endpoint " +
                           endpoint + ":");
        try {
            Service service = new Service();
            Call call = (Call) service.createCall();
            call.setOption(org.globus.wsrf.impl.security.authentication
                           .Constants.AUTHORIZATION, authorization);
            call.setTargetEndpointAddress(new java.net.URL(axisEndpoint));
            String result = (String) call.invoke("getVersion", null);
            System.out.println(result); 
        } catch (ServiceException e) {
            System.out.println("N/A");
        } catch (MalformedURLException e) {
            System.out.println("N/A");
        } catch (AxisFault e) {
            System.out.println("N/A");
        }
        System.out.println();
        
        // GT version
        System.out.println("Java WS Core Version on remote endpoint " +
                           endpoint + ":");
        try {
            Service service = new Service();
            Call call = (Call) service.createCall();
            call.setTargetEndpointAddress(new java.net.URL(gtEndpoint));
            call.setOption(org.globus.wsrf.impl.security.authentication
                           .Constants.AUTHORIZATION, authorization);
            String result = (String) call.invoke("getVersion", null);
            System.out.println(result);
        } catch (ServiceException e) {
            System.out.println("N/A");
        } catch (MalformedURLException e) {
            System.out.println("N/A");
        } catch (AxisFault e) {
            System.out.println("N/A");
        }
        System.out.println();
    }

    public static void main(String[] args) {
        
        String endpoint = null;
        Object authorization = NoAuthorization.getInstance(); 
        for (int i=0; i<args.length; i++) {
            if (args[i].equals("-help")) {
                System.out.println(OPTIONS);
                System.exit(0);
            } else if (args[i].equals("-s")) {
                if (args.length < i+1) {
                    System.out.println("-s needs a parameter");
                    System.out.println(OPTIONS);
                    System.out.println(-1);
                }
                i = i + 1;
                endpoint = args[i];
            } else if (args[i].equals("-z")) {
                if (args.length < i+1) {
                    System.out.println("-z needs a parameter");
                    System.out.println(OPTIONS);
                    System.out.println(-1);
                }
                i = i + 1;
                String value = args[i];
                if (value != null) {
                    if (value.equals("self")) {
                        authorization = SelfAuthorization.getInstance();
                    } else if (value.equals("host")) {
                        authorization = HostAuthorization.getInstance();
                    } else if (value.equals("none")) {
                        authorization = NoAuthorization.getInstance();
                    } else if (authorization == null) {
                        authorization = new IdentityAuthorization(value);
                    }
                }
            } else {
                System.out.println("Invalid paramter " + args[0]);
                System.out.println(OPTIONS);
                System.exit(-1);
            }
        }
        
        System.out.println("### OS information ###");
        System.out.println("Name    : " + System.getProperty("os.name"));
        System.out.println("Version : " + System.getProperty("os.version"));
        System.out.println("Arch.   : " + System.getProperty("os.arch"));
        
        System.out.println();
        System.out.println("### JVM information ###");
        System.out.println("Version : " + System.getProperty("java.version"));
        System.out.println("Vendor  : " + System.getProperty("java.vendor"));
        
        System.out.println();
        String javaVersion = System.getProperty("java.specification.version");
        System.out.println("### Java information ###");
        System.out.println("Version : " + javaVersion);
        
        System.out.println();
        System.out.println("### GT library version information ###");
        CheckEnvironment.displayComponentVersions(gtComponents);

        System.out.println();
        System.out.println("### 3rd party library version information ###");
        CheckEnvironment.displayComponentVersions(otherComponents);

        if (endpoint != null) {
            System.out.println();
            System.out.println("### Remote Endpoint Version Information ###");
            CheckEnvironment.displayRemoteComponentVersions(endpoint,
                                                            authorization);
        }
    }
    
    private static class ComponentVersion {
        private String name;
        private String clazz;
        private String method;
        
        public ComponentVersion(String name,
                                String clazz,
                                String method) {
            this.name = name;
            this.clazz = clazz;
            this.method = method;
        }

        public String getName() {
            return this.name;
        }

        public Object getVersion() 
            throws ClassNotFoundException,
                   NoSuchMethodException,
                   IllegalAccessException,
                   InvocationTargetException {
            Class versionClass = Class.forName(this.clazz);
            Method versionMethod = 
                versionClass.getMethod(this.method, (Class[])null);
            return versionMethod.invoke(null, (Object[])null);
        }

    }

}
